/* bios_io.h */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Console I/O routines using BIOS tty-like video service
 * and BIOS keyboard services.
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int bputc(c)
 *	Output character to screen at current cursor position.
 *
 * passed:
 *	int c := Character to display in lower 8-bits
 *
 * returns:
 *	Character displayed
 */
extern int bputc(int c);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
extern char *bputs(char *s);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int bgetc()
 *	Read one scancode/ASCII character pair from keyboard input
 *	buffer.  Wait for key to be pressed if buffer is empty.
 *
 * returns:
 *	Scancode in upper 8-bits.
 *	ASCII character in lower 8-bits.
 */
extern int bgetc();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* int kbhit()
 *	Check if there is data in the keyboard input buffer.
 *
 * returns:
 *	0 := No data available.
 *	1 := Data available.
 */
extern int kbhit();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned bputu16(n)
 *	Output unsigned decimal integer.
 *
 * passed:
 *	unsigned n := Number to display.
 *
 * returns:
 *	Number displayed.
 */
extern unsigned bputu16(unsigned n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned bputu16(n)
 *	Output unsigned long decimal integer.
 *
 * passed:
 *	unsigned long n := Number to display.
 *
 * returns:
 *	Number displayed.
 */
extern unsigned long bputu32(unsigned long n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* t_IP bputip(ip)
 *	Output IP address (1.2.3.4)
 *
 * passed:
 *	t_IP ip := IP address to display
 *
 * returns:
 *	IP address displayed
 */
extern t_IP bputip(t_IP ip);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned bputh1(n)
 *	Display one hexadecimal digit.
 *
 * passed:
 *	unsigned n := Hexadecimal digit in lower 4-bits.
 *
 * returns:
 *	Number displayed.
 */
extern unsigned bputh1(unsigned n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
extern unsigned bputh2(unsigned n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
extern unsigned bputh4(unsigned n);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
extern unsigned long bputh8(unsigned long n);
