#include "nbp.h"

extern BOOTPLAYER reply;
extern t_IP gateway;
extern int usepxe;

unsigned long kernel_addr;
unsigned long kernel_size;

unsigned long initrd_addr;
unsigned long initrd_size;

char dl_fname[256];

int
kernel_download(sip)
t_IP sip;
{
	t_DHCP_BOOTFILE *bootfile;
	t_DHCP_OVRLOAD *ovrload;
	t_PXE_MTFTP_IP *mtftp_ip;
	t_PXE_MTFTP_CPORT *mtftp_cport;
	t_PXE_MTFTP_SPORT *mtftp_sport;
	t_PXE_MTFTP_TMOUT *mtftp_tmout;
	t_PXE_MTFTP_DELAY *mtftp_delay;

	if (bootfile = find_dhcp_tag(&reply, 67, 0)) {
		strncpy(dl_fname, bootfile->buf, bootfile->len);
		dl_fname[bootfile->len] = 0;
	} else {
		if (ovrload = find_dhcp_tag(&reply, 52, 0)) {
			if (ovrload->type & 1) {
				bputs("\nPXE-Exx: No bootfile tag.\n");
				return -1;
			}
		}

		if (!*reply.bootfile) {
			bputs("\nPXE-Exx: No bootfile tag.\n");
			return -1;
		}

		strcpy(dl_fname, reply.bootfile);
	}

	kernel_size = 2 * 1024L * 1024L;

	kernel_addr = 3 * 1024L * 1024L;

	cll();
	bputs("Downloading Linux kernel image...\n");

	if ((mtftp_ip = find_pxe_tag(&reply, 1, 0)) &&
		(mtftp_cport = find_pxe_tag(&reply, 2, 0)) &&
		(mtftp_sport = find_pxe_tag(&reply, 3, 0)) &&
		(mtftp_tmout = find_pxe_tag(&reply, 4, 0)) &&
		(mtftp_delay = find_pxe_tag(&reply, 5, 0))) {

		if (tftpreadfile(kernel_addr, &kernel_size, 
			dl_fname, sip, gateway, *(t_IP *)mtftp_ip->ip, 
			mtftp_cport->port, mtftp_sport->port, 
			mtftp_tmout->tmout, mtftp_delay->delay) != 0x00) {

			bputs("\nPXE-Exx: Could not download Linux kernel image.\n");
			return -1;
		}
	} else {
		if (tftpreadfile(kernel_addr, &kernel_size, 
			dl_fname, sip, gateway, (t_IP)0, 0, 0, 0, 0) != 0x00) {

			bputs("\nPXE-Exx: Could not download Linux kernel image.\n");
			return -1;
		}
	}

#if 0
	bputs("\nkernel:  fname='"); bputs(dl_fname);
	bputs("'  fsize="); bputh8(kernel_size);
	bputs("  dl_addr="); bputh8(kernel_addr);
	bputs("\n");
#endif

	return 0;
}

int
initrd_download(sip)
t_IP sip;
{
	t_DHCP_BOOTFILE *bootfile;
	t_DHCP_OVRLOAD *ovrload;
	t_PXE_MTFTP_IP *mtftp_ip;
	t_PXE_MTFTP_CPORT *mtftp_cport;
	t_PXE_MTFTP_SPORT *mtftp_sport;
	t_PXE_MTFTP_TMOUT *mtftp_tmout;
	t_PXE_MTFTP_DELAY *mtftp_delay;

	if (bootfile = find_dhcp_tag(&reply, 67, 0)) {
		strncpy(dl_fname, bootfile->buf, bootfile->len);
		dl_fname[bootfile->len] = 0;
	} else {
		if (ovrload = find_dhcp_tag(&reply, 52, 0)) {
			if (ovrload->type & 1) {
				bputs("\nPXE-Exx: No bootfile tag.\n");
				return -1;
			}
		}

		if (!*reply.bootfile) {
			bputs("\nPXE-Exx: No bootfile tag.\n");
			return -1;
		}

		strcpy(dl_fname, reply.bootfile);
	}

	initrd_size = 4 * 1024L * 1024L;

	initrd_addr = (get_memsize() - 1024L) * 1024L - initrd_size;

	cll();
	bputs("Downloading initrd image...\n");

	if ((mtftp_ip = find_pxe_tag(&reply, 1, 0)) &&
		(mtftp_cport = find_pxe_tag(&reply, 2, 0)) &&
		(mtftp_sport = find_pxe_tag(&reply, 3, 0)) &&
		(mtftp_tmout = find_pxe_tag(&reply, 4, 0)) &&
		(mtftp_delay = find_pxe_tag(&reply, 5, 0))) {

		if (tftpreadfile(initrd_addr, &initrd_size, 
			dl_fname, sip, gateway, *(t_IP *)mtftp_ip->ip, 
			mtftp_cport->port, mtftp_sport->port, 
			mtftp_tmout->tmout, mtftp_delay->delay) != 0x00) {

			bputs("\nPXE-Exx: Could not download initrd image.\n");
			return -1;
		}
	} else {
		if (tftpreadfile(initrd_addr, &initrd_size, 
			dl_fname, sip, gateway, (t_IP)0, 0, 0, 0, 0) != 0x00) {

			bputs("\nPXE-Exx: Could not download initrd image.\n");
			return -1;
		}
	
	}

#if 0
	bputs("\ninitrd:  fname='"); bputs(dl_fname);
	bputs("'  fsize="); bputh8(initrd_size);
	bputs("  dl_addr="); bputh8(initrd_addr);
#endif

	initrd_size = 4 * 1024L * 1024L;

	return 0;
}

