#include "nbp.h"

extern BOOTPLAYER proxy;

int
do_timeout(int seconds)
{
	long lasttick;
	int ticks;
	int key;

	for (;;) {
		bputc('(');
		if (seconds > 99)
			bputh1(seconds / 100);
		if (seconds > 9)
			bputh1((seconds / 10) % 10);
		bputh1(seconds % 10);
		bputc(')');
		bputc(' ');

		for (ticks = 18; ticks--; ) {
			for (lasttick = tickcount(); lasttick == tickcount(); ) {
				if (kbhit()) {
					switch (key = bgetc()) {
					case 0x0000:	/* Ctrl+Break */
					case 0x2E03:	/* Ctrl+C */
					case 0x011B:	/* ESC */
						cll();
						return -1;

					case 0x4200:	/* F8 */
						cll();
						return 1;
					}
				}
			}
		}

		bputs(&"\b\b\b\b\b\b"[(seconds < 10) + (seconds < 100)]);

		if (!seconds--) {
			cll();
			return 0;
		}
	}
}

int
do_menu()
{
	t_PXE_BOOT_PROMPT *prompt;
	t_PXE_BOOT_MENU *menu;
	int i, n;
	int type[25];
	int longest;
	int items;
	int len;
	int top;
	int key;

	if (!(prompt = find_pxe_tag(&proxy, PXE_BOOT_PROMPT, 0)) ||
		!(menu = find_pxe_tag(&proxy, PXE_BOOT_MENU, 0))) {

		bputs("\nPXE-E74: Bad or missing PXE menu/prompt information.\n");
		return 0;
	}

	bputc('\n');

	if (prompt->dur == PXE_BOOT_PROMPT_AUTO_SELECT) {
		/* auto-select */
		return 0;
	}

	for (n = 0; n < prompt->len - 1; ++n)
		bputc(prompt->prompt[n]);

	if (prompt->dur != PXE_BOOT_PROMPT_NO_TIMEOUT) {
		switch (do_timeout(prompt->dur)) {
		case -1:
			/* Esc, Ctrl+C or Ctrl+Break */
			cll();
			bputs("PXE-MA0: Remote boot canceled by user.\n");
			return 0;

		case 0:
			/* Timer expired, auto-select first item */
			cll();
			return *(UINT16 *)menu->data;

		case 1:
			/* F8 pressed, show menu */
			cll();
		}
	}

	for (items = 0, longest = 0, n = 0; n < menu->len; ++items) {
		type[items] = *(UINT16 *)(&(menu->data[n]));

		if ((len = menu->data[n + 2]) > 70)
			len = 70;

		if (len > longest)
			longest = len;

		bputs("     ");
		for (i = 0; i < len; ++i)
			bputc(menu->data[n + 3 + i]);
		bputc('\n');

		n += 3 + menu->data[n + 2];
	}

	swapattr((top = getrow() - items), 0, (longest += 10));

	for (n = 0; ; ) {
		switch (key = bgetc()) {
		case 0x011B:	/* Esc */
		case 0x0000:	/* Ctrl+Break */
		case 0x2E03:	/* Ctrl+C */
			n = -1;
			break;

		case 0x1C0D:	/* Enter */
		case 0x1C0A:	/* Ctrl+Enter */
			break;

		case 0x0E08:	/* Backspace */
		case 0x4B00:	/* LeftArrow */
		case 0x4800:	/* UpArrow */
		case 0x0F00:	/* Shift+Tab */
		case 0x0C2D:	/* '-' */
		case 0x4A2D:	/* Keypad '-' */
			if (n) {
				swapattr(top + n, 0, longest);
				swapattr(top + --n, 0, longest);
			}

			continue;

		case 0x4D00:	/* RightArrow */
		case 0x5000:	/* DownArrow */
		case 0x0F09:	/* Tab */
		case 0x3920:	/* Space */
		case 0x0D2B:	/* '+' */
		case 0x4E2B:	/* Keypad '+' */
			if (n + 1 != items) {
				swapattr(top + n, 0, longest);
				swapattr(top + ++n, 0, longest);
			}

			continue;

		case 0x4700:	/* Home */
		case 0x4900:	/* PageUp */
			if (n) {
				swapattr(top + n, 0, longest);
				swapattr(top + (n = 0), 0, longest);
			}

			continue;

		case 0x4F00:	/* End */
		case 0x5100:	/* PageDown */
			if (n + 1 != items) {
				swapattr(top + n, 0, longest);
				swapattr(top + (n = items - 1), 0, longest);
			}

			continue;

		default:
			bputc('\r');
			bputh4(key);
			continue;
		}

		break;
	}

	gotorc(top + items, 0);
	bputc('\n');

	if (n == -1) {
		bputs("PXE-MA0: Remote boot canceled by user.\n");
		return 0;
	}

	return intswap(type[n]);
}
