/* pxe_api.h */

#ifndef _PXENV_API_H
#define	_PXENV_API_H

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
/*	Parameter structure and type definitions for PXENV API version 2.x
 *
 *	PXENV.H needs to be #included before this file.
 *
 *	None of the PXENV API services are available after the stack
 *	has been unloaded.
 */

#include "bootp.h"			/* Defines BOOTPLAYER */


/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
/* One of the following command op-codes needs to be loaded into the
 * op-code register (BX) before making a call a PXENV API service.
 */
#define PXENV_UNLOAD_STACK		0x70
#define PXENV_GET_BINL_INFO		0x71
#define	PXENV_RESTART_TFTP		0x73
#define PXENV_START_BASE		0x75
#define PXENV_STOP_BASE			0x76
#define	PXENV_GET_RAMD_INFO		0x7F


/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
/* PXENV API parameter structure typedefs.
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_UNLOAD_STACK {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT8 reserved[10];
} t_PXENV_UNLOAD_STACK;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Packet types that can be requested in the s_PXENV_GET_BINL_INFO structure. */
#define PXENV_PACKET_TYPE_DHCP_DISCOVER  1
#define PXENV_PACKET_TYPE_DHCP_ACK       2
#define PXENV_PACKET_TYPE_BINL_REPLY     3

/* Three packets are preserved and available through this interface: 1) The
 * DHCP Discover packet sent by the client, 2) the DHCP acknowledgement
 * packet returned by the DHCP server, and 3) the reply packet from the BINL
 * server.  If the DHCP server provided the image bootfile name, the
 * DHCP_ACK and BINL_REPLY packets will identical.
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_GET_BINL_INFO {
	UINT16 Status;			/* Out: See PXENV_STATUS_xxx */
					/*      constants. */
	UINT16 packet_type;		/* In: See PXENV_PACKET_TYPE_xxx */
					/*     constants */
	UINT16 buffer_size;		/* In: Size of the buffer in */
					/*     bytes.  Specifies the maximum */
					/*     amount of data that will be */
					/*     copied by the service.  A size */
					/*     of zero is valid. */
					/* Out: Amount of BINL data, in */
					/*      bytes, that was copied into */
					/*      the buffer.  For an input */
					/*      size of zero, no data will be */
					/*      copied and buffer_size will */
					/*      be set to the maximum amount */
					/*      of data available to be */
					/*      copied. */
	UINT16 buffer_offset;		/* In: 16-bit offset and segment */
	UINT16 buffer_segment;		/*     selector of a buffer where the */
					/*     requested packet will be */
					/*     copied. */
					/* Out: If buffer_size, buffer_offset */
					/*      and buffer_segment are all */
					/*      zero; buffer_offset and */
					/*      buffer_segment will be */
					/*      changed to point at the */
					/*      base code packet buffer. */
} t_PXENV_GET_BINL_INFO;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#define	s_PXENV_RESTART_TFTP	s_PXENV_TFTP_READ_FILE
#define	t_PXENV_RESTART_TFTP	t_PXENV_TFTP_READ_FILE


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_GET_RAMD_INFO {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
	UINT32 ramd_info;		/* Out: Far pointer to RAMDisk */
					/*      information structure. */
} t_PXENV_GET_RAMD_INFO;


typedef struct s_PXENV_RAMD_INFO {
	UINT16 orig_fbms;		/* Original free base memory size. */
	UINT32 ramd_addr;		/* RAMDisk physical address. */
	UINT32 orig_int13;		/* Original Int 13h ISR. */
	UINT32 orig_int15;		/* Original Int 15h ISR. */
	UINT32 fpPXEentry;		/* PXE-2.0 entry point (if not zero) */
	UINT16 dhcp_discover_off;	/* Offset & length of */
	UINT16 dhcp_discover_len;	/* DHCP discover packet. */
	UINT16 dhcp_ack_off;		/* Offset & length of */
	UINT16 dhcp_ack_len;		/* DHCP ack packet. */
	UINT16 binl_reply_off;		/* Offset & length of */
	UINT16 binl_reply_len;		/* BINL reply packet. */
	UINT16 flags;			/* RAMDisk operation flags. */
	UINT16 xms_page;		/* XMS page number of relocated */
					/* RAMDisk image.  0 == use Int 87h. */
	UINT32 xms_entry;		/* XMS API entry point. */
} t_PXENV_RAMD_INFO;


#define	PXENV_RAMD_FLAG_DISABLE		0x0001
#define	PXENV_RAMD_FLAG_PROTECTED	0x0002


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_START_BASE {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_START_BASE;


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_PXENV_STOP_BASE {
	UINT16 Status;			/* Out: PXENV_STATUS_xxx */
} t_PXENV_STOP_BASE;


#endif /* _PXENV_API_H */
