//  ARCaMP (AST Remote Control MP3 Player)
//  Copyright (C) 2000 Mike Redan (mike@jeke.fdns.net)
//  Parts based on: irmp3, Winamp IR functionality v1.0
//  Copyright (C) 1999, Andreas Neuhaus (andy@fasta.fh-dortmund.de)
//  Copyright (C) 2000 Jeremy Stoppelman (jstoppelm@yahoo.com)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "hash.h"
#include "stack.h"
#include "defs.h"
#include "launch.h"

#define BAUDRATE B1200
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1 /* POSIX compliant source */

void ExecInst(const unsigned char);

//some global variables here..makes it so I don't have to pass these things around as much..kinda dirty but
// I think it makes the code a little more readable
int random_songs = 0, pgrp;
int current_song, length2, vol_level = 50;
hash_table plist;
stack prev_list;
const char *file = default_playlist;

unsigned char readFromPort(void)
{
	//right from the serialport programming howto :)
  
	unsigned char res;
	int serialLine;
	struct termios oldtio,newtio;
	unsigned char buf[1];
	
	/* printf("-reading from port ");*/
	serialLine = open(MODEMDEVICE, O_RDWR | O_NOCTTY ); 
	if (serialLine < 0) 
	{
		perror(MODEMDEVICE); exit(-1); 
	}
	tcgetattr(serialLine,&oldtio); /* save current port settings */
	bzero(&newtio, sizeof(newtio));
	newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
	newtio.c_iflag = 0; //IGNPAR;
	newtio.c_oflag = 0;
	
     /* set input mode (non-canonical, no echo,...) */
	newtio.c_lflag = 0;
	
	newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
	newtio.c_cc[VMIN]     = 1;   /* blocking read until 1 chars received */
	
	tcflush(serialLine, TCIFLUSH);
	tcsetattr(serialLine,TCSANOW,&newtio);
	res = read(serialLine,buf,1);   /* returns after 1 chars have been input */
	tcsetattr(serialLine,TCSANOW,&oldtio);
	close (serialLine);
	/* printf ("#iraw input: %x\n", (int)buf[0]);
 */
	return buf[0];
}

int getPlayList ()
{
	FILE *fp;
	//file names can only be 1024 characters long...arbitrary number..
	char buf[1024];
	int counter = 1;
	
	if (!(fp = fopen(file, "r")))
		return 0;
	//add file names to playlist sizeof(buf)
	while (!feof(fp)) {
		if (!(fgets(buf, 1024, fp)))
			break;
		if (!(insert(counter, strdup(buf), &plist)))
			printf("error during insert!\n");
		
		counter++;
	}
	fclose(fp);
	return counter;
}

int writePlaylist ()
{
	FILE *fp;
	char *buf;
	int counter = 1;
	/*	printf ("saving %s \n", file);
*/
	if (!(fp = fopen(file, "w")))
		return 0;
	for	(counter = 1; counter != length2 -1; counter++) {
		buf = lookup(counter, &plist);
		if (buf)
			if (!(fputs(buf, fp))) 
				break;
	}
	fclose(fp);
   return counter;
}

// the real meat of the program, everything launches from here
void ExecInst(const unsigned char chRead)
{
	char buf[1024], *song_name = NULL;
	
	memset(buf, 0, 1024);
	switch (chRead)
	{
		case SHUFFLE_BTN: 
	/*	  		  printf("SHUFFLE!\n");  */
			if (random_songs) {
				/* printf("Play in sequence\n");
 */
				random_songs = 0;
			}
			else {
				/* printf("play Randomly\n");
 */
				random_songs = 1;
			}
			break;
			/*
		case VOL_UP:
		  		  printf("-VOLUME UP!\n"); 
		  
			if (vol_level <= 95)
			{
				vol_level = vol_level + 2;
				sprintf(buf, "%i", vol_level);
				volumeAdjust(buf);
								printf("from Vol_UP %s", chRead);
			}
			break;
		case VOL_DOWN:
		  		  printf("-VOLUME DOWN!\n");  
			if (vol_level >= 5) 
			{
				vol_level = vol_level - 2;
				sprintf(buf, "%i", vol_level);
				volumeAdjust (buf);
			}	
			break;
		case MUTE:
				  printf ("-mute!\n");  
			sprintf(buf, "%i", 5);
			volumeAdjust (buf);
			break;
			*/
		case PLAY_BTN:
		  /*		  printf("-play\n"); */ 
			// if song_name is NULL, then that song has been deleted...choose another one.			
			if (!(song_name = lookup(current_song, &plist)))
				ExecInst(FFW_BTN);
			snprintf(buf, 1023, song_name);
			//printf ("%s", buf);
			if (mpg123_pid)
				return;
			push (prev_list, current_song);
			playMP3(buf);
			break;
		case STOP_BTN:
		  		  /* printf("-stop \n"); */
			if (mpg123_pid) {
				kill (mpg123_pid, SIGTERM);
				mpg123_pid = 0;
				//sleep(1);
			}
			break;
		case RRW_BTN:
		  	/*	  printf("-prev\n"); */
			ExecInst(STOP_BTN);
			//get the current song out of the way,..
			current_song = pop(prev_list);
			//put the prev song in to current_song, and play it.
			current_song = pop(prev_list);
			// if we are at the beginning of the list, just go to the next song
			if (!(current_song))
				ExecInst (FFW_BTN);
			ExecInst(PLAY_BTN);
			break;
		case FFW_BTN:
		  	/*	  printf("-next\n"); */
			if (random_songs)
				current_song = 1 + (int)(((float)(length2-1)*rand())/(RAND_MAX+1.0)); 
			else 
				current_song = current_song + 1;
			//DONTPLAY gets checked in the signal handler of mpg123, if it is set, then it won't fire
			// up a new song, if it isn't set..then it will choose a new song.
			DONTPLAY	= 1;
			ExecInst(STOP_BTN);
			DONTPLAY	= 0;
			ExecInst(PLAY_BTN);
			break;
		case STAR:
			ExecInst(STOP_BTN);
			//make it so we don't hit this song when hitting the previous button			
			pop(prev_list);
			del (current_song, &plist);
			ExecInst(FFW_BTN);
			break;
		case HASH:
		  /*		  printf("write-list\n");*/
			writePlaylist ();
			break;
		case ONE:
			file = PLAYLIST_ONE;
			ExecInst(SWITCH_PLAYLIST);
			break;
		case TWO:
			file = PLAYLIST_TWO;
			ExecInst(SWITCH_PLAYLIST);
			break;
		case THREE:
			file = PLAYLIST_THREE;
			ExecInst(SWITCH_PLAYLIST);
			break;
		case FOUR:
			file = PLAYLIST_FOUR;
			ExecInst(SWITCH_PLAYLIST);
			break;
	case FIVE:
	  file = "/tmp/5.lst";
	  ExecInst(SWITCH_PLAYLIST);
	  break;
	case SIX:
	  file = "/tmp/6.lst";
	  ExecInst(SWITCH_PLAYLIST);
	  break;
	case SEVEN:
	  file = "/tmp/7.lst";
	  ExecInst(SWITCH_PLAYLIST);
	  break;
	case EIGHT:
	  file = "/tmp/8.lst";
	  ExecInst(SWITCH_PLAYLIST);
	  break;
	case NINE:
	  file = "/tmp/9.lst";
	  ExecInst(SWITCH_PLAYLIST);
	  break;
	  
		case ZERO:
			file = default_playlist;
			ExecInst(SWITCH_PLAYLIST);
			break;
			
		case SWITCH_PLAYLIST:
			ExecInst(STOP_BTN);
			free_list(prev_list);
			free_table(&plist, NULL);
			construct_table(&plist,1500);
			length2 = getPlayList();
			if (!(length2))
			{
				printf ("error loading playlist:%s \n", file);
				exit(-1);
			}
			ExecInst(FFW_BTN);
			break;
		default:
		  /*			printf ("unknown :%i:\n", (int)chRead);
*/
		 
		  
			break;			
	}
}

int main (void)
{
	unsigned char	chRead = 0;
	
	//initialize everything
	prev_list = initStack();
	srand(time(NULL));
	construct_table(&plist,1500);
	random_songs = 1;
	length2 = getPlayList ();
	mpg123_pid = 0;
	volumeAdjust("50");
	if (length2 == 0) {
		printf("error loading playlist %s...getting out", file);
		return 0;
	}
	//ExecInst (NEXT);
	// execute forever

	ExecInst(PLAY_BTN);
	while (1)
	{
		// Read actual instruction
	  chRead = readFromPort();
	       
		if (chRead == 0xA0){ // && chRead != 0xAA)
			chRead = readFromPort(); //Instruction to do
			if (((chRead == VOL_UP) || (chRead == VOL_DOWN)) || ((int)chRead > 130))
				while(((chRead == VOL_UP) || (chRead == VOL_DOWN)) || ((int)chRead > 130))
				{
				  // No need for volume
				  //ExecInst(chRead);				
					chRead = readFromPort();
				}
			/* printf("[ %x ]", (int) chRead); */
			ExecInst(chRead);
			
			}
	}
}	
