#include "nbp.h"

#define BCAST_IP	0xFFFFFFFFLU
#define MCAST_IP_LOW	0xE0000000LU
#define MCAST_IP_HIGH	0xEFFFFFFFLU

extern BOOTPLAYER discover;
extern BOOTPLAYER dhcpack;
extern BOOTPLAYER proxy;
extern BOOTPLAYER reply;

extern t_IP gateway;

int
_discover(t_IP *ip, int sport, int cport, int ipcnt, t_IP *iplist, int _timeout, int type)
{
	t_IP dip;
	t_IP sip;
	long last_tick;
	int bufsize;
	int dport;
	int timeout;
	int pktsize;
	int retry;
	int ticks;
	int n;

	pktsize = (sizeof(BOOTPLAYER) - BOOTP_DHCPVEND) + 312;

	discover.seconds = 0;

	if (udpopen() != PXENV_STATUS_SUCCESS)
		return -1;

	if (_timeout) {
		timeout = _timeout;
		retry = 1;
	} else {
		timeout = 1;
		retry = 4;
	}

	for ( ; retry--; ++timeout) {
		bputc('.');

		if (udpwrite(ip, gateway, sport, cport, _DS(), &discover, pktsize) != PXENV_STATUS_SUCCESS) {
			udpclose();

			if (longswap(*ip) >= MCAST_IP_LOW && longswap(*ip) <= MCAST_IP_HIGH) {
				/* We might be running on a 99b, or earlier,
				 * PXE ROM.  Multicast discovery will not work.
				 */

				return 0;
			}

			bputs("\nPXE-Exx: UDP write error.\n");
			return -1;
		}

		for (ticks = (timeout * 182L) / 10, last_tick = tickcount(); ticks; ) {
			if (kbhit()) {
				switch (bgetc()) {
				case 0x011B:	/* Esc */
				case 0x0000:	/* Ctrl+Break */
				case 0x2E03:	/* Ctrl+C */
					bputs("\nPXE-MA0: Remote boot aborted by user.\n");
					udpclose();
					return -1;
				}
			}

			if (last_tick != tickcount()) {
				--ticks;
				last_tick = tickcount();
			}

			dip = 0;
			sip = 0;
			dport = 0;
			bufsize = sizeof reply;

			if (udpread(&dip, &sip, &dport, &bufsize, _DS(), &reply) != PXENV_STATUS_SUCCESS)
				continue;

			if (!bufsize)
				continue;

			switch (dport) {
			case INTSWAP(BOOTP_CPORT):
			case INTSWAP(PXE_BINL_PORT):
				break;

			default:
				continue;
			}

			if (dip != LONGSWAP(BCAST_IP) && dip != *(t_IP *)dhcpack.yip) {
				continue;
			}

			if (reply.opcode != BOOTP_REP) {
				continue;
			}

			if (*(t_IP *)reply.yip) {
				continue;
			}

			if (!find_dhcp_tag(&reply, DHCP_BOOTFILE, 0)) {
				if (find_dhcp_tag(&reply, DHCP_OVRLOAD, 0)) {
					continue;
				}

				if (!*reply.bootfile) {
					continue;
				}
			}

			if (ipcnt && iplist) {
				for (n = 0; n < ipcnt; ++n) {
					if (sip == iplist[n])
						break;
				}

				if (n == ipcnt) {
					continue;
				}
			}

			udpclose();
			return 1;
		}
	}
			
	udpclose();
	return 0;
}

int
multicast_discover(t_IP *ip, int ipcnt, t_IP *iplist, int type)
{
	cll();
	bputs("MD ");
	bputip(*ip);
	bputc(' ');

	return _discover(ip, PXE_BINL_PORT, BOOTP_CPORT, ipcnt, iplist, 0, type);
}

int
broadcast_discover(int ipcnt, t_IP *iplist, int type)
{
	t_IP ip;

	ip = LONGSWAP(BCAST_IP);

	cll();
	bputs("BD ");

	return _discover(&ip, BOOTP_SPORT, BOOTP_CPORT, ipcnt, iplist, 0, type);
}

int
unicast_discover(int ipcnt, t_IP *iplist, int type)
{
	int timeout;
	int n;
	int r;

	if (ipcnt == 1) {
		cll();
		bputs("UD ");
		bputip(iplist[n]);
		bputc(' ');
	}

	for (timeout = 1; timeout <= 4; ++timeout) {
		for (n = 0; n < ipcnt; ++n) {
			if (ipcnt > 1) {
				cll();
				bputs("UD ");
				bputip(iplist[n]);
				bputc(' ');

				if (timeout > 1)
					bputs(&"..."[4 - timeout]);
			}

			if (r = _discover(&iplist[n], PXE_BINL_PORT, BOOTP_CPORT, 0, 0, timeout, type))
				return r;
		}
	}

	return 0;
}

int
do_discover(int type, int layer)
{
	t_DHCP_MSGTYPE *msgtype;
	t_DHCP_SRVID *srvid;
	t_PXE_BOOT_ITEM *bootitem;
	t_PXE_DISCOVERY_CONTROL *discctrl;
	t_PXE_MCAST_DISCOVERY_ADDR *discaddr;
	t_PXE_BOOT_SERVERS *srvlist;
	t_IP *iplist;
	t_IP _iplist[2];
	int ipcnt;
	int n;

	if (!(msgtype = find_dhcp_tag(&discover, DHCP_MSGTYPE, 0))) {
		bputs("\nPXE-Exx: Missing DHCP message type tag.\n");
		return -2;
	}

	msgtype->type = DHCP_REQUEST;

	*(t_IP *)discover.cip = *(t_IP *)dhcpack.yip;

	if (!(bootitem = find_pxe_tag(&discover, PXE_BOOT_ITEM, 0))) {
		if (!(bootitem = add_pxe_tag(&discover, PXE_BOOT_ITEM, sizeof(t_PXE_BOOT_ITEM)))) {
			bputs("\nPXE-Exx: Cannot write PXE_BOOT_ITEM tag.\n");
			return -2;
		}
	}

	bootitem->type = intswap(type);
	bootitem->layer = intswap(layer);

	if (layer) {
		if (srvid = find_dhcp_tag(&proxy, DHCP_SRVID, 0))
			iplist = &srvid->ip;
		else
			iplist = proxy.sip;

		if (!*iplist) {
			bputs("\nPXE-Exx: Cannot find boot server IP address.\n");
			return -2;
		}

		return unicast_discover(1, iplist);
	}

	discctrl = find_pxe_tag(&proxy, PXE_DISCOVERY_CONTROL, 0);
	discaddr = find_pxe_tag(&proxy, PXE_MCAST_DISCOVERY_ADDR, 0);

	iplist = 0;
	ipcnt = 0;

	if (discctrl) {
		if (discctrl->flag & PXE_DISCOVERY_CONTROL_SRVLIST_ONLY) {
			if (!(srvlist = find_pxe_tag(&proxy, PXE_BOOT_SERVERS, 0))) {
				bputs("\nPXE-Exx: Missing boot server list.\n");
				return -2;
			}

			for (n = 0; ; ) {
				if (type == intswap(*(UINT16 *)&srvlist->data[n]))
					break;

				if (n + srvlist->data[n + 2] * 4 >= srvlist->len) {
					bputs("\nPXE-Exx: Missing boot server type.\n");
					return -2;
				}
			}

			iplist = &srvlist->data[n + 3];
			ipcnt = srvlist->data[n + 2];
		}
	}

#if 0
	if (!ipcnt || !iplist) {
		ipcnt = 2;
		iplist = _iplist;

		iplist[0] = LONGSWAP(0x01020304L);
		iplist[1] = LONGSWAP(0xCFAF2C82L);
	}
#endif

#if 1
	if (discaddr) {
		if (!discctrl || !(discctrl->flag & PXE_DISCOVERY_CONTROL_MCAST_DISABLE)) {
			switch (multicast_discover(discaddr->ip, ipcnt, iplist)) {
			case 1: /* found */
				return 1;

			case -1: /* abort */
				return -1;
			}
		}
	}
#endif

#if 1
	if (!discctrl || !(discctrl->flag & PXE_DISCOVERY_CONTROL_BCAST_DISABLE)) {
		switch (broadcast_discover(ipcnt, iplist)) {
		case 1: /* found */
			return 1;

		case -1: /* abort */
			return -1;
		}
	}
#endif

	if (ipcnt && iplist)
		return unicast_discover(ipcnt, iplist);

	return 0;
}
