/* linux.c */

#include "nbp.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* External variables
 */
extern int usepxe;

extern unsigned long initrd_addr;
extern unsigned long initrd_size;

extern unsigned SetupSeg;

extern t_PXE _pxe;
extern t_PXENV _pxenv;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Global variables
 */
BOOTPLAYER discover;
BOOTPLAYER dhcpack;
BOOTPLAYER proxy;
BOOTPLAYER reply;

t_IP gateway;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* External procedures
 */

extern int kernel_download(t_IP sip);
extern int initrd_download(t_IP sip);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
int
hang()
{
	bputs("\nCannot continue, please reboot.\n");

	for (;;)
		;
}

int
do_timeout(int seconds)
{
	long lasttick;
	int ticks;
	int key;

	for (;;) {
		bputc('(');
		if (seconds > 99)
			bputh1(seconds / 100);
		if (seconds > 9)
			bputh1((seconds / 10) % 10);
		bputh1(seconds % 10);
		bputc(')');
		bputc(' ');

		for (ticks = 18; ticks--; ) {
			for (lasttick = tickcount(); lasttick == tickcount(); ) {
				if (kbhit()) {
					key = bgetc();
					return 1;
				}
			}
		}

		bputs(&"\b\b\b\b\b\b"[(seconds < 10) + (seconds < 100)]);

		if (!seconds--) {
			cll();
			return 0;
		}
	}
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* unsigned nbpentry(pxeptr, pxenvptr)
 *	C entry point for PXE bootstrap programs.
 *
 * passed:
 *	t_PXE far *pxeptr := Far pointer to !PXE structure
 *	t_PXENV far *pxenvptr := Far pointer to PXENV+ structure
 *
 * returns:
 *	16-bit status that will be returned to PXE boot ROM
 */
unsigned
nbpentry(unsigned long pxeptr, unsigned long pxenvptr)
{
	t_DHCP_ROUTER *router;
	t_DHCP_SRVID *srvid;
	t_PXE_BOOT_ITEM *bootitem;
	t_IP sip;
	int type;
	int layer;
	char cmdline[255];
	char * chptr = cmdline;
	int key;
	int where;

	cmdline[0] = '\0';

	bputs("\nIntel Linux NBP, Beta-3 (build 003)\n\n");

	if (get_memsize() < 6 * 1024) {
		bputs("Not enough memory to remote boot Linux.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Verify PXE structures and initialize global PXE variables.
	 */

	if (!pxeinit(pxeptr, pxenvptr)) {
		bputs("PXE-E81: !PXE structure is invalid.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Get cached packets from PXE boot ROM
	 */

	if (!get_cached_packet(PXENV_PACKET_TYPE_DHCP_DISCOVER, &discover)) {
		bputs("Could not get cached DHCP Discover packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!get_cached_packet(PXENV_PACKET_TYPE_DHCP_ACK, &dhcpack)) {
		bputs("Could not get cached DHCP Ack packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!get_cached_packet(PXENV_PACKET_TYPE_BINL_REPLY, &proxy)) {
		bputs("Could not get cached Proxy packet.\n");
		return PXENV_STATUS_FAILURE;
	}

	bputs("Press any key to enter kernel parameters... ");
	if (do_timeout(5) == 1) {
		bputc('\n');
		bputs("Enter kernel parameters: ");
		where = getcol();
		for (key = bgetc(); key != 0x1C0D; key = bgetc()) {
			switch (key) {
				case 0x0E08:	/* Backspace */
					if (chptr == cmdline) break;
					chptr--;
					gotorc(getrow(), 
						chptr - cmdline + where);
					bputc(' ');
					gotorc(getrow(), 
						chptr - cmdline + where);
					break;

				default:
					*chptr++ = key & 0xFF;
					bputc(key & 0xFF);
					break;
			}
		}
		*chptr = '\0';
		bputc('\n');
	}
	else
	{
 	  strncpy(cmdline,"ks console=ttyS0,115200",23);
	}
	bputc('\n');

	/*
	 * Initialize default gateway for UDP and TFTP API calls.
	 */

	if (usepxe) {
		/*
		 * Default gateway is 0.0.0.0 for PXE 2.0 ROMs.
		 */

		gateway = LONGSWAP(0);
	} else {
		/*
		 * Get default gateway from DHCP Ack packet
		 */

		if (router = find_dhcp_tag(&dhcpack, DHCP_ROUTER, 0))
			gateway = *(t_IP *)router->ip;
		else
			gateway = *(t_IP *)dhcpack.gip;

		/*
		 * If there is no default gateway in the DHCP Ack packet,
		 * but there is one in the Proxy packet, use the one in
		 * Proxy packet.
		 */

		if (!gateway) {
			if (router = find_dhcp_tag(&proxy, DHCP_ROUTER, 0))
				gateway = *(t_IP *)router->ip;
			else
				gateway = *(t_IP *)proxy.gip;
		}
	}

	/*
	 * Get boot server IP address.
	 */

	if (srvid = find_dhcp_tag(&proxy, DHCP_SRVID, 0))
		sip = *(t_IP *)&srvid->ip;
	else
		sip = proxy.sip;

	if (!sip) {
		bputs("PXE-Exx: Cannot find boot server IP address.\n");
		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Get previous boot item type & layer.
	 */

	if (!(bootitem = find_pxe_tag(&proxy, PXE_BOOT_ITEM, 0))) {
		bputs("PXE-Exx: Cannot find PXE_BOOT_ITEM (43,71) tag.\n");
		return PXENV_STATUS_FAILURE;
	}

	if (!(type = intswap(bootitem->type))) {
		bputs("PXE-Exx: PXE_BOOT_ITEM type cannot be zero.\n");
		return PXENV_STATUS_FAILURE;
	}

	layer = intswap(bootitem->layer) & 0x7FFF;

	/*
	 * Discover selected boot server type.  The first discovery
	 * layer is always zero.  Non-zero discovery layers will
	 * only do unicast discovery to the PXE boot server that is
	 * in the cached proxy packet.
	 */

	/*
	 * For this NBP we will assume that the next layer number is
	 * going to be our Linux kernel ...
	 */

	if (do_discover(type, layer + 1) != 1) {
		/* 
		 * A discovery error occured.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_FAILURE;
	}

	/*
	 * Download kernel
	 */

	if (kernel_download(sip)) {
		/*
		 * Kernel could not download (or the download was
		 * aborted).  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_FAILURE;
	}

	/*
	 * ... and that the layer number after the Linux kernel
	 * layer is our uncompressed initrd image.  If there is
	 * no initrd image, we will boot assuming the kernel was
	 * built/setup for a NFSroot boot.
	 */

	switch (do_discover(type, layer + 2)) {
	case -2: /* error */
		/* 
		 * A discovery error occured.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_FAILURE;

	case -1: /* abort */
		/*
		 * Discovery aborted by user.  Return to PXE boot ROM.
		 */

		return PXENV_STATUS_SUCCESS;

	case 0: /* not found */
		/*
		 * initrd image not found - assume NFS-root boot
		 */

		initrd_size = 0;
		initrd_addr = 0;

		break;

	case 1: /* found */
#if 1
		/*
		 * Download initrd image
		 */

		if (initrd_download(sip)) {
			/*
			 * Initrd image could not download (or the download
			 * was aborted).  Return to PXE boot ROM.
			 */

			return PXENV_STATUS_FAILURE;
		}

		if (initrd_addr < 5 * 1024L * 1024L) {
			bputs("\nNot enough memory to remote boot Linux+initrd.\n");
			return PXENV_STATUS_FAILURE;
		}
#else
		initrd_size = 0;
		initrd_addr = 0;
#endif
	}

	/*
	 * Shutdown and unload PXE base-code.
	 * From this point on, we cannot return control to PXE.
	 */

	if (!unload_pxe())
		hang();

	/*
	 * Initialize boot sector and setup parameters
	 */

	if (!prepare_bootsect_setup(cmdline))
		return PXENV_STATUS_FAILURE;

	/*
	 * Run Linux setup.  If setup.s changes in the kernel loader,
	 * the setup.s file in linux.0 needs to be updated.
	 */

	Setup();

	hang();
}
