#include "nbp.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
extern unsigned long kernel_addr;
extern unsigned long kernel_size;
extern unsigned long initrd_addr;
extern unsigned long initrd_size;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
unsigned long BootSectAddr;
unsigned long SetupAddr;
unsigned BootSectSeg;
unsigned SetupSeg;

UINT32 data, dataaddr, cksumaddr, _cksum, cnt;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_bootsect_table {
	unsigned char reserved1[32];
	unsigned cmdline_magic;
	unsigned cmdline_offset;
	unsigned char reserved2[461];
	unsigned char setup_sects;	/* 497 : number of setup sectors */
	unsigned root_flags;		/* 498 : ? */
	unsigned syssize;		/* 500 : ? */
	unsigned swap_dev;		/* 502 : ? */
	unsigned ram_flag;		/* 504 : RAMDisk load flags */
					/*       8000h - ask for RAMDisk */
					/*       4000h - load RAMDisk */
					/*       07FFh - starting block number mask */
	unsigned vid_mode;		/* 506 : ? */
	unsigned root_dev;		/* 508 : ? */
	unsigned boot_flag;		/* 510 : diskette boot signature 0xAA55 */
} BOOTSECT;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
typedef struct s_setup_table {
	unsigned char jump[2];		/*   0h : jump past data */
	unsigned char setup_header[4];	/*   2h : "HdrS" */
	unsigned version;		/*   6h : Header format version 0x0201 */
	unsigned long rmode_switch;	/*   8h : ? */
	unsigned long start_sys_seg;	/*   Ch : not used */
	unsigned char type_of_loader;	/*  10h : etherboot 0x40 */
	unsigned char load_flags;	/*  11h : loaded_high 0x01 */
	unsigned setup_move_size;	/*  12h : not used */
	unsigned long code32_start;	/*  14h : 0x100000 */
	unsigned long ramdisk_addr;	/*  18h : RAMDisk address */
	unsigned long ramdisk_size;	/*  1Ch : RAMDisk size in bytes */
	unsigned long bootsect_load;	/*  20h : not used */
	unsigned heap_end_ptr;		/*  24h : not used */
	unsigned long base_mem_size;	/*  26h : Base memory size in bytes */
} SETUP;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
UINT16
get_extmem()
{
#asm
	mov	ax, #$8800
	int	#$15
#endasm
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
int
prepare_bootsect_setup(char * cmdline)
{
	BOOTSECT bootsect;
	SETUP setup;
	unsigned long Ksource;
	unsigned long Kdest;
	unsigned long Kcount;
	unsigned count;
	int n;

	/*
	 * Compute BootSectAddr and SetupAddr.
	 * This is where the bootsect and setup sectors
	 * will be copied from the kernel image.
	 */

	BootSectSeg = ((fbms() << 6) - 0x0E00) & 0xF000;
	BootSectAddr = (unsigned long)BootSectSeg << 4;

	SetupSeg = BootSectSeg + 0x20;
	SetupAddr = (unsigned long)SetupSeg << 4;

#if 0
	bputs("\nBootSectSeg="); bputh4(BootSectSeg);
	bputs("\nBootSectAddr="); bputh8(BootSectAddr);
	bputs("\nSetupSeg="); bputh4(SetupSeg);
	bputs("\nSetupAddr="); bputh8(SetupAddr);
#endif

	/*
	 * Copy the boot sector and setup images from the kernel image.
	 */

	if (n = pmode_copy(BootSectAddr, kernel_addr, 256)) {
		bputs("\nBIOS protected mode memory copy failed: ");
		bputh4(n);
		bputc('\n');

		return 0;
	}

	_fmemcpy(MK_FP(_SS(), &bootsect), MK_FP(BootSectSeg, 0), sizeof bootsect);

#if 0
	if (n = pmode_copy(SetupAddr, kernel_addr + 512, bootsect.setup_sects * 256))
#else
	if (n = pmode_copy(SetupAddr, kernel_addr + 512, 0x4000))
#endif
	{
		bputs("\nBIOS protected mode memory copy failed: ");
		bputh4(n);
		bputc('\n');

		return 0;
	}

	_fmemcpy(MK_FP(_SS(), &setup), MK_FP(SetupSeg, 0), sizeof setup);

	/*
	 * Get kernel source & destination addresses.
	 * Copy kernel image from download location to startup location.
	 */

	Ksource = kernel_addr + ((bootsect.setup_sects + 1) * 512L);
	Kdest = 0x100000L;
	Kcount = ((kernel_size - ((bootsect.setup_sects + 1) * 512L)) + 1) >> 1;

#if 0
	bputs("\nkernel:  dest="); bputh8(Kdest);
	bputs("  src="); bputh8(Ksource);
	bputs("  size="); bputh8(Kcount << 1);
#endif

#if 0
	{
		dataaddr = ((UINT32)_DS() << 4) + (UINT16)&data;

		bputs("\naddress-  checksum + --data-- = checksum");
		bputs("\n--------  -------- + -------- = --------");

		cksumaddr = Ksource;
		_cksum = 0;
		cnt = 0x4000;

		while (cnt--) {
			if (!(cnt > 10 && cnt < 0x4000 - 10)) {
				bputc('\n');
				bputh8(cksumaddr);
				bputs("  ");
				bputh8(_cksum);
				bputs(" + ");
			}

			if (pmode_copy(dataaddr, cksumaddr, 2)) {
				bputs("\npmode_copy()!\n");
				return -1;
			}

			if (!(cnt > 10 && cnt < 0x4000 - 10)) {
				bputh8(data);
				bputs(" = ");
			}

			_cksum += data;

			if (!(cnt > 10 && cnt < 0x4000 - 10))
				bputh8(_cksum);

			cksumaddr += 4;
		}
	}
#endif

	while (Kcount) {
#if 0
		bputs("\nKdest="); bputh8(Kdest);
		bputs("  Ksource="); bputh8(Ksource);
		bputs("  Kcount="); bputh8(Kcount);
#endif

		count = (unsigned)((Kcount >= 0x8000L) ? 0x8000L : Kcount);

		pmode_copy(Kdest, Ksource, count);

		Kcount -= count;
		Ksource += 0x10000L;
		Kdest += 0x10000L;
	}

	kernel_addr = 0x100000L;

	/*
	 * Fill in setup fields.
	 */

	setup.type_of_loader = 0x40;	/* ETHERBOOT, ver 0 */

	if (setup.load_flags & 0x01)
		setup.code32_start = 0x100000L;
	else
		setup.code32_start = 0x1000L;

	/*
	 * Initialize boot device for initrd, if necessary.
	 */

	bootsect.cmdline_magic = 0xA33F;
	bootsect.cmdline_offset = 36;
	strcpy(bootsect.reserved2, cmdline);
	
	if (initrd_addr && initrd_size) {
		/*
		 * /dev/ram major and minor device numbers
		 */

		bootsect.root_dev = 0x0101;

		/*
		 *
		 */

		bootsect.ram_flag = 0x4000;

		/*
		 * Location and size of initrd image.
		 */

		setup.ramdisk_addr = initrd_addr;
		setup.ramdisk_size = initrd_size;
	}

	/*
	 * Prepare hardware and fill in initialization parameters.
	 */

	prepare_hardware(&count, &((UINT8 *)&bootsect)[0x1FF]);

	n = 2 + (UINT8 *)&bootsect;	/* This places the offset of */
					/* bootsect[2] into BX. */

	*(UINT16 *)&((UINT8 *)&bootsect)[0x02] = get_extmem();

	_fmemcpy(MK_FP(_SS(), &((UINT8 *)&bootsect)[0x80]),
		MK_FP(0, 0x41 * 4), 0x10);

	if (count) {
		_fmemcpy(MK_FP(_SS(), &((UINT8 *)&bootsect)[0x90]),
			MK_FP(0, 0x46 * 4), 0x10);
	} else {
		memset(&((UINT8 *)&bootsect)[0x90], 0, 0x10);
	}

	memset(&((UINT8 *)&bootsect)[0x40], 0, 0x12);

	((UINT8 *)&bootsect)[0] = getcol();
	((UINT8 *)&bootsect)[1] = getrow();
	((UINT8 *)&bootsect)[6] = getmode();
	*(UINT16 *)&((UINT8 *)&bootsect)[4] = getvstuff();

	((UINT8 *)&bootsect)[7] = getcols();
	((UINT8 *)&bootsect)[0x0E] = getrows();
	((UINT8 *)&bootsect)[0x0F] = 1;
	*(UINT16 *)&((UINT8 *)&bootsect)[0x0A] = 0;

	_fmemcpy(MK_FP(_SS(), &((UINT8 *)&bootsect)[0x10]),
		MK_FP(0x40, 0x85), 2);

	/*
	 * See if the non-640k patch flag is set in the load flags.
	 * If the flag is not set, we will need to relocate the
	 * boot and setup sectors to segment 0x9000 before we start
	 * the setup code.
	 */

	if (setup.load_flags & 0x02)
		setup.base_mem_size = fbms() * 1024L;

	/*
	 * Copy boot sector and setup structures back to
	 * their respective segments.
	 */

	_fmemcpy(MK_FP(BootSectSeg, 0), MK_FP(_SS(), &bootsect), sizeof bootsect);

	_fmemcpy(MK_FP(SetupSeg, 0), MK_FP(_SS(), &setup), sizeof setup);

	/*
	 * Relocate the boot/setup images if this is a non-640k
	 * patched kernel.
	 */

	if (!(setup.load_flags & 0x02)) {
		_fmemcpy(MK_FP(0x9000, 0), MK_FP(BootSectSeg, 0),
			(bootsect.setup_sects + 1) * 512);

		BootSectSeg = 0x9000;
		BootSectAddr = (unsigned long)BootSectSeg << 4;

		SetupSeg = 0x9020;
		SetupAddr = (unsigned long)SetupSeg << 4;
	}

#if 0
	bputs("\n<Enter>\n");
	bgetc();
#endif

	return 1;
}
