#include "nbp.h"

t_PXE _pxe;

t_PXENV _pxenv;

int usepxe = 0;

int
_pxe_check()
{
	if (_pxe.StructLength >= 8 &&
		_pxe.Signature[0] == '!' && _pxe.Signature[1] == 'P' &&
		_pxe.Signature[2] == 'X' && _pxe.Signature[3] == 'E') {

		if (!cksum(&_pxe, sizeof(t_PXE))) {
			usepxe = 1;
			return 1;
		}
	}

	return 0;
}

int
_pxenv_check()
{
	if (_pxenv.StructLength >= 8 &&
		_pxenv.Signature[0] == 'P' && _pxenv.Signature[1] == 'X' &&
		_pxenv.Signature[2] == 'E' && _pxenv.Signature[3] == 'N' &&
		_pxenv.Signature[4] == 'V' && _pxenv.Signature[5] == '+') {

		if (cksum(&_pxenv, _pxenv.StructLength))
			return 0;
	} else {
		return 0;
	}

	/*
	 * If we get here, we have a good PXENV+ structure.
	 * Check to see if it points to a !PXE structure.
	 */

	_fmemcpy(MK_FP(_DS(), &_pxe), _pxenv.RuntimePtr, sizeof _pxe);

	_pxe_check();

	return 1;
}

int
pxeinit(UINT32 pxeptr, UINT32 pxenvptr)
{
	_fmemcpy(MK_FP(_DS(), &_pxe), pxeptr, sizeof _pxe);
	_fmemcpy(MK_FP(_DS(), &_pxenv), pxenvptr, sizeof _pxenv);

	/*
	 * If the !PXE structure is happy, just use it!
	 */

	if (_pxe_check())
		return 1;

	/*
	 * Do two checks for the PXENV+ structure.  There are a couple of
	 * buggy ROM versions out there that have the PXENV+ pointer in
	 * different places.
	 */

	/*
	 * First check the PXENV+ pointer where it would be in a good ROM.
	 * If this check passes, we are done.
	 */

	if (_pxenv_check())
		return 1;

	/*
	 * Next, copy the PXENV+ structure from the !PXE pointer that
	 * was passed in and check it.
	 */

	_fmemcpy(MK_FP(_DS(), &_pxenv), pxeptr, sizeof _pxenv);

	if (_pxenv_check())
		return 1;

	/*
	 * Nothin' good was found, return an error.
	 */

	return 0;
}

