! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! This module must be first and must be linked to run at offset 0:7C00h.
!

extern _nbpentry
.global __pxe
.global __pxenv
.global _usepxe

.text

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int main(unsigned long pxeptr)
!	Enter here from PXE ROM.  Save register contents.
!	Setup DS, SS & SP for C.
!
! passed:
!	ES:BX := Far pointer to PXENV+ structure
!	BP[6] := Far pointer to !PXE structure
!
! returns:
!	AX := Return status from nbpentry()
!
export auto_start
auto_start:

export _main
_main:
	push	bp
	mov	bp, sp

	push	bx
	push	cx
	push	dx
	push	si
	push	di

	push	ds
	push	es

	push	cs
	pop	ds

	mov	dx, [bp + 8]
	mov	cx, [bp + 6]

	call	_switch_stack	! trashes di & si

	push	es
	push	bx

	push	dx
	push	cx

	call	_nbpentry

	add	sp, #8

	call	_switch_stack	! trashes di & si

	pop	es
	pop	ds

	pop	di
	pop	si
	pop	dx
	pop	cx
	pop	bx

	pop	bp
	retf

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int bputc(int c)
!	Display character using BIOS TTY-like services.
!
! passed:
!	c := ASCII character in lower 8-bits
!
! returns:
!	Character passed.
!
export _bputc
_bputc:
	push	bp
	mov	bp, sp

	push	bx
	push	cx
	push	dx
	push	di
	push	si

	mov	ax, [bp + 4]

	mov	ah, #$0E
	mov	bx, #$700
	int	#$10

	mov	ax, [bp + 4]
	cmp	al, #$0A
	jne	bputc_notlf

	mov	ax, #$0E0D
	mov	bx, #$700
	int	#$10

	mov	ax, [bp + 4]

bputc_notlf:
	xor	ah, ah

	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int kbhit()
!	Check if key has been pressed.
!
! passed:
!	nothing
!
! returns:
!	Zero - no key has been pressed.
!	Non-zero - key has been pressed.
!
export _kbhit
_kbhit:
	mov	ah, #1
	int	#$16

	mov	ax, #0
	jz	kbhit_nokey

	inc	ax
kbhit_nokey:
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int bgetc()
!	Return keyboard scan code and ASCII key.
!	Will wait for key if one has not been pressed.
!
! passed:
!	nothing
!
! returns:
!	scan code: in upper 8-bits
!	ASCII char: in lower 8-bits
!
export _bgetc
_bgetc:
	mov	ah, #0
	int	#$16
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int _SS()
!	Return contents of SS register.
!
! passed:
!	nothing
!
! returns:
!	SS
!
export __SS
__SS:
	mov	ax, ss
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int _DS()
!	Return contents of DS register.
!
! passed:
!	nothing
!
! returns:
!	DS
!
export __DS
__DS:
	mov	ax, ds
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! int _fmemcpy(UINT32 t, UINT32 f, UINT16 s)
!	Copy up to 64K-1 bytes from (t) to (f).
!	(t) and (f) are seg:off addresses created by MK_FP()
!
! passed:
!	t: To address
!	f: From address
!	s: Byte count
!
! returns:
!	Bytes copied
!
export __fmemcpy
__fmemcpy:
	push	bp
	mov	bp, sp

	push	cx
	push	di
	push	si
	push	ds
	push	es

	mov	cx, [bp + 12]
	jcxz	fmemcpy_done

	les	di, [bp + 4]
	lds	si, [bp + 8]
	cld
	rep
	movsb

fmemcpy_done:
	mov	ax, [bp + 12]
	sub	ax, cx

	pop	es
	pop	ds
	pop	si
	pop	di
	pop	cx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _pxe
_pxe:
	push	bp
	mov	bp, sp

	push	bx
	push	cx
	push	dx
	push	di
	push	si
	push	ds
	push	es

	cmp	_usepxe, #0
	je	pxe_usepxenv

	push	[bp + 8]
	push	[bp + 6]
	push	[bp + 4]

	callf	[__pxe + 16]

	add	sp, #6

	jmp	pxe_done

pxe_usepxenv:
	mov	bx, [bp + 4]
	les	di, [bp + 6]

	callf	[__pxenv + 10]

pxe_done:
	pop	es
	pop	ds
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _tickcount
_tickcount:
	push	ds

	mov	ax, #$40
	mov	ds, ax
	mov	ax, $6C
	mov	dx, $6E

	pop	ds
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _swapattr
_swapattr:
	push	bp
	mov	bp, sp

	push	bx
	push	cx
	push	dx
	push	ds

	mov	cx, [bp + 8]
	jcxz	swapattr_done

	mov	ax, #$B000
	mov	ds, ax

	mov	bx, [bp + 4]
	mov	ax, #160
	mul	bx
	mov	bx, [bp + 6]
	shl	bx, #1
	add	bx, ax
	inc	bx

swapattr_loop:
	mov	al, [bx]
	mov	ah, al
	ror	al, #4
	and	ax, #$8877
	or	al, ah
	mov	[bx], al

	mov	al, [bx + $8000]
	mov	ah, al
	ror	al, #4
	and	ax, #$8877
	or	al, ah
	mov	[bx + $8000], al

	add	bx, #2
	loop	swapattr_loop

swapattr_done:
	pop	ds
	pop	dx
	pop	cx
	pop	bx

	xor	ax, ax

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getrow
_getrow:
	push	bx
	push	cx
	push	dx

	mov	ah, #$03
	xor	bh, bh
	int	#$10

	mov	al, dh
	xor	ah, ah

	pop	dx
	pop	cx
	pop	bx

	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getcol
_getcol:
	push	bx
	push	cx
	push	dx

	mov	ah, #$03
	xor	bh, bh
	int	#$10

	mov	al, dl
	xor	ah, ah

	pop	dx
	pop	cx
	pop	bx

	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getrows
_getrows:
	push	bx
	push	cx
	push	dx
	push	di
	push	si
	push	ds
	push	es

	sub	sp, #64

	mov	si, ss
	mov	es, di
	mov	ds, di
	mov	di, sp

	mov	ax, #$1B00
	xor	bx, bx
	int	#$10
	jc	getrows1

	cmp	al, #$1B
	jne	getrows1

	mov	ax, [di + $22]
	jmp	getrows_done

getrows1:
	mov	ax, #25

getrows_done:
	add	sp, #64

	pop	es
	pop	ds
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	ret


! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getcols
_getcols:
	push	bx
	push	cx
	push	dx
	push	di
	push	si
	push	ds
	push	es

	sub	sp, #64

	mov	si, ss
	mov	es, di
	mov	ds, di
	mov	di, sp

	mov	ax, #$1B00
	xor	bx, bx
	int	#$10
	jc	getcols1

	cmp	al, #$1B
	jne	getcols1

	mov	ax, [di + 5]
	jmp	getcols_done

getcols1:
	mov	ah, #$0F
	int	#$10
	jc	getcols2

	shr	ax, #8
	jmp	getcols_done

getcols2:
	mov	ax, #80

getcols_done:
	add	sp, #64

	pop	es
	pop	ds
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getmode
_getmode:
	push	bx

	mov	ah, #$0F
	int	#$10

	pop	bx

	xor	ah, ah
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _getvstuff
_getvstuff:
	push	bx

	mov	ah, #$0F
	xor	bx, bx
	int	#$10

	mov	ax, bx

	pop	bx

	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _gotorc
_gotorc:
	push	bp
	mov	bp, sp

	push	bx
	push	cx
	push	dx

	mov	ah, #$02
	xor	bh, bh
	mov	dh, [bp + 4]
	mov	dl, [bp + 6]
	int	#$10

	pop	dx
	pop	cx
	pop	bx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _fbms
_fbms:
	push	ds

	mov	ax, #$40
	mov	ds, ax
	mov	ax, $13

	pop	ds
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _setfbms
_setfbms:
	push	bp
	mov	bp, sp

	push	ds

	mov	ax, #$40
	mov	ds, ax

	mov	ax, [bp + 4]
	xchg	ax, $13

	pop	ds

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _switch_stack
_switch_stack:
	pop	ax	! return address

	mov	si, origSS
	mov	di, origSP

	mov	origSS, ss
	mov	origSP, sp

	cli
	mov	ss, si
	mov	sp, di
	sti

	push	ax

	xor	ax, ax
	xor	si, si
	xor	di, di
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _get_memsize
_get_memsize:
	push	bx
	push	cx
	push	di
	push	si
	push	ds
	push	es

	mov	ax, #$E801	! Get memory size
	clc
	int	#$15
	jnc	getmsz_e801

	mov	ax, #$DA88	! Get memory size
	clc
	int	#$15
	jnc	getmsz_da88

	mov	ah, #$8A	! Get memory size
	clc
	int	#$15
	jnc	getmsz_8a

	mov	ah, #$88	! Get memory size
	clc
	int	#$15
	jnc	getmsz_88

	xor	dx, dx
	xor	ax, ax

getmsz_done:
	pop	es
	pop	ds
	pop	si
	pop	di
	pop	cx
	pop	bx

	ret

getmsz_88:
	or	ax, ax
	jz	getmsz_64m

	xor	dx, dx

getmsz_8a:
	add	ax, #1024	! Add in 1M
	adc	dx, #0

	jmp	getmsz_done

getmsz_64m:
	mov	dx, #1
	xor	ax, ax

	jmp	getmsz_done

getmsz_da88:
	xor	dh, dh
	mov	dl, cl
	mov	ax, bx

	jmp	getmsz_8a

getmsz_e801:
	xor	dx, dx
	or	bx, bx
	jz	getmsz_88

	mov	ax, #64
	mul	bx

	add	ax, #16384
	adc	dx, #0

	jmp	getmsz_done

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
gdt:	dw	0, 0, 0, 0	! not used
	dw	0, 0, 0, 0	! gdt
	dw	0, 0, 0, 0	! source (from)
	dw	0, 0, 0, 0	! destination (to)
	dw	0, 0, 0, 0	! zero
	dw	0, 0, 0, 0	! zero

export _pmode_copy
_pmode_copy:
	push	bp
	mov	bp, sp

	!
	! Save registers
	!

	push	bx
	push	cx
	push	dx
	push	di
	push	si
	push	ds
	push	es

	!
	! DS:[BX] := Address of zero filled GDT
	!

	mov	bx, cs
	mov	ds, bx
	mov	bx, #gdt

	mov	cx, #4 * 6	! number of words to zero fill
pmcp_zero_gdt:
	mov	word [bx], #0
	add	bx, #2
	loop	pmcp_zero_gdt

	mov	bx, #gdt

	!
	! Fill in address of GDT in GDT[1]
	!

	add	bx, #8

	mov	ax, cs
	mov	dx, #16
	mul	dx
	add	ax, #gdt
	adc	dx, #0

	mov	[bx + 2], ax
	mov	[bx + 4], dl
	mov	[bx + 7], dh
	mov	[bx + 0], #8 * 6 - 1	! (sizeof gdt) - 1
	mov	byte [bx + 5], #$93

	!
	! Fill in source address in GDT[2]
	!

	add	bx, #8

	mov	ax, [bp + 8]
	mov	[bx + 2], ax
	mov	ax, [bp + 10]
	mov	[bx + 4], al
	mov	[bx + 7], ah
	mov	word [bx + 0], #$FFFF
	mov	byte [bx + 5], #$93

	!
	! Fill in destination address in GDT[3]
	!

	add	bx, #8

	mov	ax, [bp + 4]
	mov	[bx + 2], ax
	mov	ax, [bp + 6]
	mov	[bx + 4], al
	mov	[bx + 7], ah
	mov	word [bx + 0], #$FFFF
	mov	byte [bx + 5], #$93

	!
	! Copy memory
	!

	mov	ax, #$8700
	mov	cx, [bp + 12]
	mov	si, cs
	mov	es, si
	mov	si, #gdt
	clc
	int	#$15

	shr	ax, #8

	!
	! Restore registers and return
	!

	pop	es
	pop	ds
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
export _prepare_hardware
_prepare_hardware:
	push	bp
	mov	bp, sp

	!
	! Save registers
	!

	push	bx
	push	cx
	push	dx
	push	di
	push	si
	push	ds
	push	es

	!
	! Set *hdcount
	!

	mov	bx, [bp + 4]
	mov	word [bx], #0

	mov	ax, #$1500	! Check for second hard drive
	mov	dl, #$81
	int	#$13
	jc	prephw_nohd1

	cmp	ah, #3
	jne	prephw_nohd1

	mov	bx, [bp + 4]
	mov	word [bx], #1

prephw_nohd1:

	!
	! Reset controllers
	!

	xor	ax, ax
	mov	dx, #$80
	int	#$13

	!
	! Max out keyboard repeat rate
	!

	mov	ax, #$0305
	xor	bx, bx
	int	#$16

	!
	! Check for PS/2 pointing device
	!

	int	#$11

	and	al, #$04
	jz	prephw_nops2

	mov	al, #$AA

prephw_nops2:
	mov	bx, [bp + 6]	! *mouse
	mov	[bx], al

	!
	! Restore registers and return
	!

	pop	es
	pop	ds
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx

	pop	bp
	ret

! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
.data
origSP:	dw	$7C00
origSS:	dw	0

.bss

